# Localized

Localized is a Laravel package which integrates localized routes in your application so that it seems seamless. It is a successor to my package [loki](https://github.com/laravelista/loki), which was my first attempt to solve this problem.

[![Become a Patron](https://img.shields.io/badge/Become%20a-Patron-f96854.svg?style=for-the-badge)](https://www.patreon.com/laravelista)

## Overview

Almost every website that I build is multilingual. In the beginning I used [mcamara/laravel-localization](https://github.com/mcamara/laravel-localization), but when I discovered that it does not support route caching (It does support it now) I created my own package [laravelista/loki](https://github.com/laravelista/loki). At first it seemed great, but when I started working with it more and more I started noticing its drawbacks.

This package "Localized" is a successor to loki in every way. I have taken everything that I liked from loki, improved upon it and integrated it more tightly into Laravel, so that now the whole experience seems seamless as you will see in the example bellow.

Localized leverages [Psalm](https://psalm.dev/) to achieve 100% type inference; It is "totallyTyped" (level 1). It runs only on PHP 8+ and it has tests so that I can detect bugs more easily (Ideally this should be at 100% test coverage). Code is styled using [`php-cs-fixer`](https://cs.symfony.com/) according to the PSR1 and PSR12.

## Features

- Seamless integration with Laravel routes.
- Easy and simple configuration.
- Route caching out of the box.
- Support for localized routes.
- Support for route parameter localization.
- Does not affect the behaviour of the rest of your Laravel application.
- Allows defining custom locales.

## Installation

From the command line:

```
composer require laravelista/localized
```

### Configuration

Publish the config file with:

```
php artisan vendor:publish --provider="Laravelista\Localized\ServiceProvider" --tag=config
```

You will find it under `config/localized.php`.

#### `locales` [array]

Locale names (codes) can be whatever you want.

_example._ en-GB, hr-HR, en-US, english, croatian, german, de, fr, ...

```
'locales' => ['hr', 'en'],
```

#### `defaultLocale` [string]

Must be from one of the locales defined in `locales`. 

Must be defined here because the one in `config/app.php` is changed when the app locale changes.

```
'defaultLocale' => 'en',
```

## Macros

This package adds three macros which you can use when defining your routes.

- `->identifier('id')` - This macro stores the route id which is shared for routes with same content but different locale.
- `->locale('locale')` - This macro stores the locale for the route and applies needed middleware. If the locale is omitted that means that that route is for the default locale.
- `Route::locale('locale')->group(function() {})` - This macro sets the locale and middleware for all routes defined in the group.

## Example

Your Localized config file:

```php
'defaultLocale' => 'en',

'locales' => [
    'en',
    'hr'
]
```

Your routes file:

```php
Route::get('/articles/{slug}', [ArticleController::class, 'show'])->name('articles.show')->identifier('articles.show');

Route::get('/clanci/{slug}', [ArticleController::class, 'show'])->name('hr.articles.show')->identifier('articles.show')->locale('hr');
```


### Page navigation

To navigate to the current locale route, in your views:

```php
<a href="{{ route(Localized::route('articles.show'), $article->slug) }}">{{ __('View Article') }}</a>
```

To navigage to a specific locale route, in your views:

```php
<a href="{{ route(Localized::route('articles.show', 'hr'), $article->slug) }}">{{ __('View Article') }}</a>
```

As you can see the only difference is that instead of calling `route('articles.show', $article->slug)` with `articles.show` as the route name,
you are calling `Localized::route('articles.show')` with the route name.


### Language picker

Somewhere in your layout view, eg. `layouts/base.blade.php`:

```php
<ul>
    @foreach(config('localized.locales') as $locale)
        <li>
            <a href="{{ Localized::get($locale, $localizedParameters ?? []) }}">
                {{ __($locale) }}
            </a>
        </li>
    @endforeach
</ul>
```

If you are using [astrotomic/laravel-translatable](https://github.com/Astrotomic/laravel-translatable) for handling model field translations,
in your view, eg. `articles/show.blade.php`:

```php
@php
$localizedParameters = [];

foreach(config('localized.locales') as $locale) {
  $localizedParameters[$locale] = [
    'slug' => $article->{"slug:{$locale}"}
  ];
}
@endphp

@extends('layouts.base', [
    'localizedParameters' => $localizedParameters
])
```

This seems very verbose at first, but it allows a high level of customization in regard to
obtaining model field translations.


## Development

```bash
# Install dependencies
composer install

# Run tests
vendor/bin/phpunit
vendor/bin/phpunit --test-coverage=html ./coverage
vendor/bin/phpunit --testdox

# Run Psalm
vendor/bin/psalm

# Format code (php-cs-fixer)
vendor/bin/php-cs-fixer fix
```

## Research

- [Tell Google about localized versions of your page](https://developers.google.com/search/docs/advanced/crawling/localized-versions)
- [laravelista/loki](https://github.com/laravelista/loki)
- [mcamara/laravel-localization](https://github.com/mcamara/laravel-localization)

## Sponsors

I would like to extend my thanks to all my sponsors for funding my open-source journey. If you are interested in becoming a sponsor, please visit the [Backers page](https://mariobasic.com/backers) on my website.

## Contributing

Thank you for considering contributing to Localized! The contribution guide can be found [Here](https://mariobasic.com/contributing).

## Code of Conduct

In order to ensure that the open-source community is welcoming to all, please review and abide by the [Code of Conduct](https://mariobasic.com/code-of-conduct).

## License

Localized is created by [Mario Bašić](https://mariobasic.com) under the [Sponsorware license](https://github.com/sponsorware/docs).

By being granted access to this repository you agree to not distribute or make profit from this source code.

_Sponsorware is a release strategy for open-source software that enables developers to be compensated for their open-source work with fewer downsides than traditional open-source funding models._